package tutorial;

import com.nativelibs4java.opencl.*;
import com.nativelibs4java.opencl.CLPlatform.DeviceFeature;
import com.nativelibs4java.util.*;
import java.io.File;
import java.io.IOException;
import java.nio.FloatBuffer;
import org.apache.commons.math.complex.Complex;
import org.apache.commons.math.transform.FastFourierTransformer;

public class DFT_float {

    final CLQueue queue;
    final CLContext context;
    final CLProgram program;
    final CLKernel kernel;

    public DFT_float(CLQueue queue) throws IOException, CLBuildException {
        this.queue = queue;
        this.context = queue.getContext();

        String source = IOUtils.readText(new File("/Users/ochafik/Prog/Java/versionedSources/nativelibs4java/trunk/libraries/OpenCL/JavaCLTutorial/src/main/opencl/tutorial/DiscreteFourierTransformProgram_float.cl"));//DFT.class.getClassLoader().getResource("tutorial.DiscreteFourierTransformProgram.cl"));
        program = context.createProgram(source);
        kernel = program.createKernel("dft");
    }

    /**
     * Method that takes complex values in input (sequence of pairs of real and imaginary values) and
     * returns the Discrete Fourier Transform of these values if forward == true or the inverse
     * transform if forward == false.
     */
    public synchronized FloatBuffer dft(FloatBuffer in, boolean forward) {
        assert in.capacity() % 2 == 0;
        int length = in.capacity() / 2;

        // Create an input CLBuffer that will be a copy of the NIO buffer :
        CLFloatBuffer inBuf = context.createFloatBuffer(CLMem.Usage.Input, in, true); // true = copy

        // Create an output CLBuffer :
        CLFloatBuffer outBuf = context.createFloatBuffer(CLMem.Usage.Output, length * 2);

        // Set the args of the kernel :
        kernel.setArgs(inBuf, outBuf, length, forward ? 1 : -1);

        // Ask for `length` parallel executions of the kernel in 1 dimension :
        CLEvent dftEvt = kernel.enqueueNDRange(queue, new int[]{ length });

        // Return an NIO buffer read from the output CLBuffer :
        return outBuf.read(queue, dftEvt);
    }

    /// Wrapper method that takes and returns float arrays
    public float[] dft(float[] complexValues, boolean forward) {
        FloatBuffer outBuffer = dft(FloatBuffer.wrap(complexValues), forward);
        float[] out = new float[complexValues.length];
        outBuffer.get(out);
        return out;
    }
    static float[] toReal(float[] in) {
        float[] inReal = new float[in.length / 2];
        for (int i = 0; i < inReal.length; i++)
            inReal[i] = in[i / 2];

        return inReal;
    }
    static float[] createTestFloatData(int n) {
        //int n = 1024 * 1024;
        float[] in = new float[2 * n];

        for (int i = 0; i < n; i++) {
            in[i * 2] = 1 / (float) (i + 1);
            in[i * 2 + 1] = 0;
        }
        return in;
    }
}
